@@ -271,9 +271,9 @@ impl EspNetif {
                     InterfaceStack::Sta => _g_esp_netif_netstack_default_wifi_sta,
                     InterfaceStack::Ap => _g_esp_netif_netstack_default_wifi_ap,
                     InterfaceStack::Eth => _g_esp_netif_netstack_default_eth,
-                    #[cfg(esp_idf_version = "4.4")]
-                    InterfaceStack::Ppp => panic!("Not yet supported on ESP-IDF 4.4"),
-                    #[cfg(not(esp_idf_version = "4.4"))]
+                    #[cfg(any(esp_idf_version = "4.4", esp_idf_version_major = "5"))]
+                    InterfaceStack::Ppp => panic!("Not yet supported on ESP-IDF 4.4 & ESP-IDF 5.X"),
+                    #[cfg(not(any(esp_idf_version = "4.4", esp_idf_version_major = "5")))]
                     InterfaceStack::Ppp => _g_esp_netif_netstack_default_ppp,
                 }
             },
