@@ -1,6 +1,8 @@
 use core::{cmp, convert::TryInto, ptr, time::Duration};
 
 extern crate alloc;
+use alloc::borrow::ToOwned;
+use alloc::boxed::Box;
 use alloc::sync::Arc;
 use alloc::vec;
 
@@ -886,7 +888,11 @@ impl Wifi for EspWifi {
 
         let mut ap_infos_raw: vec::Vec<wifi_ap_record_t> =
             vec::Vec::with_capacity(total_count as usize);
-        unsafe { ap_infos_raw.set_len(total_count as usize) };
+        #[allow(clippy::uninit_vec)]
+        // ... because we are filling it in on the next likem and only reading the initialized members
+        unsafe {
+            ap_infos_raw.set_len(total_count as usize)
+        };
 
         let real_count = self.do_get_scan_infos(&mut ap_infos_raw)?;
 
